"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PreRuntimeDetectionPlugin = void 0;
const utils_1 = require("@morjs/utils");
const constants_1 = require("../constants");
const SOURCETYPE_CORE_CORRESPONDENCE = {
    origin: { app: 'App', page: 'Page', component: 'Component' },
    alipay: { app: 'aApp', page: 'aPage', component: 'aComponent' },
    wechat: { app: 'wApp', page: 'wPage', component: 'wComponent' }
};
/**
 * 进行编译的相关检查，对不符合要求的进行警告
 * 1. 运行时 core 和 配置的 sourceType 类型是否一致
 */
class PreRuntimeDetectionPlugin {
    constructor() {
        this.name = 'PreRuntimeDetectionPlugin';
    }
    apply(runner) {
        runner.hooks.beforeRun.tap(this.name, () => {
            // 非 compile 命令 跳过
            if (runner.commandName !== constants_1.COMPILE_COMMAND_NAME)
                return;
            runner.hooks.scriptParser.tap(this.name, (transformers, options) => {
                var _a, _b;
                const { sourceType } = runner.userConfig;
                const { entryName, entryType, content: fileContent } = options.fileInfo;
                const rightCore = (_a = SOURCETYPE_CORE_CORRESPONDENCE[sourceType]) === null || _a === void 0 ? void 0 : _a[entryType];
                // 与配置 DSL 不一致的运行时
                const oppositeCore = (_b = SOURCETYPE_CORE_CORRESPONDENCE[this.oppositeSourceType(sourceType)]) === null || _b === void 0 ? void 0 : _b[entryType];
                if (rightCore &&
                    oppositeCore &&
                    fileContent &&
                    fileContent.includes(oppositeCore)) {
                    this.isCoreSameSourceType({
                        transformers,
                        sourceType,
                        entryName,
                        rightCore,
                        oppositeCore
                    });
                }
                return transformers;
            });
        });
    }
    oppositeSourceType(sourceType) {
        return sourceType === 'alipay'
            ? 'wechat'
            : sourceType === 'wechat'
                ? 'alipay'
                : null;
    }
    /**
     * 运行时 core 和 配置的 sourceType 类型是否一致
     * @param transformers
     */
    isCoreSameSourceType({ transformers, sourceType, entryName, rightCore, oppositeCore }) {
        transformers.before.push((0, utils_1.tsTransformerFactory)((node) => {
            if (!node)
                return node;
            if (utils_1.typescript.isExpressionStatement(node) &&
                utils_1.typescript.isCallExpression(node.expression) &&
                utils_1.typescript.isIdentifier(node.expression.expression)) {
                const { escapedText } = node.expression.expression;
                if (escapedText && oppositeCore === escapedText) {
                    utils_1.logger.warn(`检测到运行时与用户配置 sourceType 类型 ${sourceType} 不匹配\n` +
                        `文件路径：${entryName} \n` +
                        `建议修改 ${escapedText} 为 ${rightCore}，否则可能引发非预期的问题`);
                }
            }
            return node;
        }));
    }
}
exports.PreRuntimeDetectionPlugin = PreRuntimeDetectionPlugin;
//# sourceMappingURL=preRuntimeDetectionPlugin.js.map